# This submodule name would change to an author's desired name.
# This submodule must be wrapped within an author's toplevel module.
module SocketHandler

  @@DEBUG = false

  @@queue = nil
  @@socket_spy = nil

  class DictSpy < Sketchup::EntityObserver
  
    attr_writer(:debug) # creates @debug var and debug=() method.
  
    ### PRIVATE METHODS
    
    private
  
    def initialize
      @processed = []
      @debug = false
    end

    def handle_messages(dict)
      if dict.size > 0
        model = dict.model
        dict.each { | key, message |
          unless @processed.include?(key)
            process_message(key,message,model)
            @processed << key
          end
        }
      end
    end
    
    def process_message(key,message,model)
      puts("Socket message processed.\n  key: '#{key.to_s}'\n  message: '#{message.to_s}'\n") if @debug
      #
      # This is where the message processing conditional statement goes.
      #
    end
    
    ### CALLBACKS
    
    public

    def onChangeEntity(dict)
      if dict.class==(Sketchup::AttributeDictionary)
        handle_messages(dict) if dict.size > 0
        @processed.each {|key|
          UI.start_timer(1.0,false) {
            @processed.delete(key)
            dict.delete_key(key) 
          }
        }
      else
        puts("#{dict.to_s} is not a Sketchup::AttributeDictionary object.")
      end
    end

    def onEraseEntity(dict)
      puts("#{dict.name} AttributeDictionary removed from the model.")
    end

  end # Observer class


  ### MODULE METHODS

  def self.debug(arg=true)
    @@socket_spy.debug= @@DEBUG =( arg ? true : false )
  end

  def self.message_count()
    @@queue.size
  end
  
  def self.test_message(key,msg)
    msg = msg.to_s
    @@queue[key]=msg
  end
  
  def self.queue()
    @@queue
  end
  
  def self.queue_hash()
    hash = {}
    @@queue.to_a.each {|e| hash[e.first]=e.last }
    return hash
  end
  
  
  ### MODULE INIT ROUTINE
  begin
    model = Sketchup.active_model
    @@queue = model.attribute_dictionary('SocketQueue',true)
    @@socket_spy = DictSpy.new() unless @@socket_spy
    @@socket_spy.debug= @@DEBUG
    @@queue.add_observer(@@socket_spy)
  end
  
end # module